<?php
namespace Ep\Gath;

use Ep\Core\AbstractFactory;
use Ep\Gath\WordPress\OrderTypeProvider;
use Ep\Gath\WordPress\PriceProvider;
use Ep\Gath\WordPress\ProductProvider;
use Ep\Gath\WordPress\StockProvider;
use Ep\WordPress\Settings\SettingsProviderInterface;

/**
 * Konfiguracija Gath modula.
 *
 * @package Ep\Gath
 * @author Elektronika Perhoč <support@ep.hr>
 * @link https://bitbucket.org/elektronika-perhoc/ep-webshop
 */
class ConfigProvider implements SettingsProviderInterface
{
	/**
	 * @return array
	 */
	public function __invoke()
	{
		$gathConfig = $this->getDefaults()['ep_gath'];

		if (function_exists('get_option')) {
			$option = get_option('ep_gath', []);

			if (!isset($option['timeout']) || empty($option['timeout'])) {
				$option['timeout'] = $gathConfig['timeout'];
			}

			foreach ($gathConfig as $name => $value) {
				if (array_key_exists($name, $option)) {
					$gathConfig[$name] = $option[$name];
				}
			}
		}

		$metadataProviders = [];
		$wordpressServices = [
			'GathImageProvider'
		];
		$httpClients = [];

		if ($gathConfig['enabled'] === '1') {
			$metadataProviders[] = 'GathOrderTypeProvider';

			if (in_array('prices', $gathConfig['modules'])) {
				$metadataProviders[] = 'GathPriceProvider';
			}

			if (in_array('products', $gathConfig['modules'])) {
				$metadataProviders[] = 'GathProductProvider';
			}

			if (in_array('stock', $gathConfig['modules'])) {
				$metadataProviders[] = 'GathStockProvider';
			}

			$httpClients['http.gath'] = [
				'cache' => 'cache.gath',
				'base_uri' => $gathConfig['base_uri'],
				'timeout' => intval($gathConfig['timeout']),
			];
		}

		return [
			'dependencies' => $this->getDependencyConfig(),
			'hydrators' => $this->getHydratorConfig(),
			'caches' => $this->getCacheConfig(),
			'gath' => $gathConfig,
			'http_clients' => $httpClients,
			'wordpress' => [
				'services' => $wordpressServices,
				'metadata_providers' => $metadataProviders,
			],
		];
	}

	/**
	 * @return array
	 */
	public function getCacheConfig()
	{
		return [
			'cache.gath' => [
				'adapter' => defined('WP_DEBUG') && WP_DEBUG ? 'null' : 'filesystem',
			],
		];
	}

	/**
	 * Vraća konfiguraciju za ServiceManager
	 *
	 * @return array
	 */
	public function getDependencyConfig()
	{
		return [
			'aliases' => [
				'Gath' => GathService::class,
				'GathRestAdapter' => Adapter\RestAdapter::class,
				'GathOrderTypeProvider' => OrderTypeProvider::class,
				'GathImageProvider' => Wordpress\ImageProvider::class,
				'GathPriceProvider' => PriceProvider::class,
				'GathProductProvider' => ProductProvider::class,
				'GathStockProvider' => StockProvider::class,
			],
			'factories' => [
				GathService::class => GathServiceFactory::class,
				Adapter\RestAdapter::class => Adapter\RestAdapterFactory::class,
                OrderTypeProvider::class => AbstractFactory::class,
				Wordpress\ImageProvider::class => WordPress\ImageProviderFactory::class,
                PriceProvider::class => AbstractFactory::class,
                ProductProvider::class => AbstractFactory::class,
                StockProvider::class => AbstractFactory::class,
			],
		];
	}

	public function getHydratorConfig()
	{
		return [
			'factories' => [
				Entity\Discount::class => Hydrator\DiscountHydratorFactory::class,
				Entity\Price::class    => Hydrator\PriceHydratorFactory::class,
				Entity\Product::class => Hydrator\ProductHydratorFactory::class,
				Entity\ProductBrand::class => Hydrator\ProductBrandHydratorFactory::class,
				Entity\ProductImage::class => Hydrator\ProductImageHydratorFactory::class,
				Entity\ProductShippingClass::class => Hydrator\ProductShippingClassHydratorFactory::class,
				Entity\Rebate::class => Hydrator\RebateHydratorFactory::class,
				Entity\Stock::class    => Hydrator\StockHydratorFactory::class,
			],
		];
	}

	/**
	 * Vraća zadane postavke koje provider pruža.
	 *
	 * @return array
	 */
	public function getDefaults()
	{
		return [
			'ep_gath' => [
				'enabled' => '0',
				'base_uri' => '',
				'images_base_uri' => '',
				'timeout' => 2,
				'modules' => [],
				'dontPreloadGrouped' => '0',
			],
		];
	}

	/**
	 * Vraća forme za postavke.
	 *
	 * @return array
	 */
	public function getForms()
	{
		return [
			'ep_gath' => 'settings-gath',
		];
	}

	/**
	 * Vraća tabove u control panelu koje ovaj provider definira.
	 *
	 * @return array
	 */
	public function getTabs()
	{
		return [[
			'id' => 'gath',
			'title' => 'Gath',
			'optionKeys' => [
				'ep_gath',
			]
		]];
	}
}