<?php
namespace Ep;

use Automatic_Upgrader_Skin;
use Ep\WordPress\Service\AbstractWordPressService;
use Language_Pack_Upgrader;
use WC_Admin_Notices;
use WC_Install;
use WC_Shipping_Zone;
use WC_Tax;

/**
 * Instalira i konfigurira WooCommerce.
 *
 * @package Ep\WooCommerce
 * @author Elektronika Perhoč <support@ep.hr>
 * @link https://bitbucket.org/elektronika-perhoc/ep-webshop
 */
class Installer extends AbstractWordPressService
{
	/**
	 * Zadane postavke za WooCommerce.
	 *
	 * @var array
	 */
    private $woocommerceDefaults = [
		'woocommerce_default_country' => 'HR',
        'woocommerce_currency' => 'HRK',
        'woocommerce_currency_pos' => 'right_space',
        'woocommerce_price_decimal_sep' => ',',
        'woocommerce_price_num_decimals' => '2',
        'woocommerce_price_thousand_sep' => '.',
        'woocommerce_calc_taxes' => 'yes',
        'woocommerce_prices_include_tax' => 'yes',
        'woocommerce_weight_unit' => 'kg',
        'woocommerce_dimension_unit' => 'cm',
	];

    /**
     * Zadana stopa poreza.
     *
     * @var array
     */
    private $woocommerceDefaultTaxRate = [
        'tax_rate_country' => 'HR',
        'tax_rate_state' => '',
        'tax_rate' => '25.0000',
        'tax_rate_priority' => 1,
        'tax_rate_compound' => 0,
        'tax_rate_shipping' => 1,
        'tax_rate_order' => 0,
        'tax_rate_class' => '',
    ];

    /**
     * @var array
     */
    private $randomImages;

	/**
	 * Inicijalizacija.
	 */
	public function attach(): void
	{
        if (false === get_option('ep_version')) {
            $this->setup();
        }
	}

    /**
     * Admin
     */
    public function attachAdmin(): void
    {
        $this->removeNotices();
    }

    /**
	 * Instalira i konfigurira sve što trebamo.
	 */
	private function setup()
	{
		require_once ABSPATH . '/wp-admin/includes/plugin.php';

		if (!is_plugin_active('woocommerce/woocommerce.php')) {
            $this->setupWooCommerce();
            $this->setupTheme();
            $this->updateTranslations();
            $this->setupContent();
        }

        update_option('ep_version', EP_WEBSHOP_VERSION);
		flush_rewrite_rules();
	}

	/**
	 * Instalira i konfigurira WooCommerce.
	 */
	private function setupWooCommerce()
	{
        activate_plugin('woocommerce/woocommerce.php');

		WC_Install::create_pages();
		wc()->init();

		foreach ($this->woocommerceDefaults as $option => $value) {
		    update_option($option, $value);
        }

        WC_Tax::_insert_tax_rate($this->woocommerceDefaultTaxRate);

		$location = wc_format_country_state_string($this->woocommerceDefaults['woocommerce_default_country']);

		$zone = new WC_Shipping_Zone();
		$zone->set_zone_order(0);
		$zone->add_location($location['country'], 'country');
		$zone->set_zone_name($zone->get_formatted_location());
		$zone->add_shipping_method('free_shipping');
		$zone->save();

		WC_Admin_Notices::remove_notice('install');
	}

    /**
     * Instalira i konfigurira zadanu temu.
     */
	private function setupTheme()
    {
        switch_theme('flatsome');
        update_option('envato_setup_complete', time());
    }

    /**
     * Ažurira i dohvaća potrebne prijevode.
     */
    private function updateTranslations()
    {
        require_once ABSPATH . '/wp-admin/includes/file.php';
        require_once ABSPATH . '/wp-admin/includes/misc.php';
        require_once ABSPATH . '/wp-admin/includes/class-wp-upgrader.php';
        require_once ABSPATH . '/wp-admin/includes/class-language-pack-upgrader.php';

        $upgraderSkin = new Automatic_Upgrader_Skin();
        $upgrader = new Language_Pack_Upgrader($upgraderSkin);
        $upgrader->bulk_upgrade();
    }

    /**
     * Briše defaultni sadržaj i dodaje naš.
     */
    private function setupContent()
    {
        wp_delete_comment(1, true);
        wp_delete_post(1, true);
        wp_delete_post(2, true);

        $frontPageId = wp_insert_post([
            'post_title' => 'Naslovnica',
            'post_type' => 'page',
        ]);

        update_option('page_on_front', $frontPageId);
        update_option('show_on_front', 'page');


    }

    /**
     * Dodaje nekakve medijske zapise.
     */
    private function setupMedia()
    {

    }

    /**
     * Uklanja dosadne, beskorisne poruke iz admina.
     */
    private function removeNotices()
    {
//        $flatsome = \TGM_Plugin_Activation::get_instance();
//        remove_action('admin_notices', [$flatsome, 'notices']);
    }
}