<?php
namespace Ep\WooCommerce\View;

use Ep\WordPress\Service\AbstractWordPressService;
use WC_Product;
use WC_Product_Grouped;
use WC_Product_Variable;
use Zend\Stdlib\ArrayUtils;

/**
 * Obrasci za dodavanje u košaricu.
 *
 * @package Ep\WooCommerce\View
 * @author Elektronika Perhoč <support@ep.hr>
 * @link https://bitbucket.org/elektronika-perhoc/ep-webshop
 */
class AddToCart extends AbstractWordPressService
{
	public function attach(): void
	{
		remove_action('woocommerce_simple_add_to_cart', 'woocommerce_simple_add_to_cart', 30);
		add_action('woocommerce_simple_add_to_cart', [$this, 'renderSimple'], 30);

		remove_action('woocommerce_grouped_add_to_cart', 'woocommerce_grouped_add_to_cart', 30);
		add_action('woocommerce_grouped_add_to_cart', [$this, 'renderGrouped'], 30);

		remove_action('woocommerce_variable_add_to_cart', 'woocommerce_variable_add_to_cart', 30);
		add_action('woocommerce_variable_add_to_cart', [$this, 'renderVariable'], 30);
	}

	/**
	 * Renderira obrazac za jednostavni proizvod.
	 *
	 * @param int|null $productId
	 */
	public function renderSimple($productId = null)
	{
		/** @var WC_Product $product */
		global $product;

		if (!empty($productId)) {
			$product = wc_get_product($productId);
		}

		ob_start();
		woocommerce_quantity_input([
			'min_value'   => apply_filters('woocommerce_quantity_input_min', $product->get_min_purchase_quantity(), $product),
			'max_value'   => apply_filters('woocommerce_quantity_input_max', $product->get_max_purchase_quantity(), $product),
			'input_value' => isset($_POST['quantity']) ? wc_stock_amount($_POST['quantity']) : $product->get_min_purchase_quantity(),
		]);
		$quantityInput = ob_get_clean();

		$templateVars = ArrayUtils::merge($this->getTemplateVars($product), [
			'quantity_input' => $quantityInput,
		]);

		echo $this->getView()->render('@woocommerce/single-product/add-to-cart/simple.twig', $templateVars);
	}

	/**
	 * Renderira obrazac za grupirani proizvod.
	 *
	 * @param int|null $productId
	 */
	public function renderGrouped($productId = null)
	{
		/** @var WC_Product_Grouped $product */
		global $product;

		if (!empty($productId)) {
			$product = wc_get_product($productId);
		}

		$templateVars = ArrayUtils::merge($this->getTemplateVars($product), [
			'children' => $this->renderGroupedChildren($product),
		]);

		echo $this->getView()->render('@woocommerce/single-product/add-to-cart/grouped.twig', $templateVars);
	}

	/**
	 * Renderira obrazac za varijabilni proizvod.
	 *
	 * @param int|null $productId
	 */
	public function renderVariable($productId = null)
	{
		/** @var WC_Product_Variable $product */
		global $product;

		if (!empty($productId)) {
			$product = wc_get_product($productId);
		}

		wp_enqueue_script('wc-add-to-cart-variation');

		$variationThreshold = apply_filters('woocommerce_ajax_variation_threshold', 30, $product);
		$hasVariations = count($product->get_children()) <= $variationThreshold;

		ob_start();
		woocommerce_quantity_input([
			'min_value'   => apply_filters('woocommerce_quantity_input_min', $product->get_min_purchase_quantity(), $product),
			'max_value'   => apply_filters('woocommerce_quantity_input_max', $product->get_max_purchase_quantity(), $product),
			'input_value' => isset($_POST['quantity']) ? wc_stock_amount($_POST['quantity']) : $product->get_min_purchase_quantity(),
		]);
		$quantityInput = ob_get_clean();

		$templateVars = ArrayUtils::merge($this->getTemplateVars($product), [
			'attributes' => $this->renderVariableAttributes($product),
			'available' => $hasVariations,
			'quantity_input' => $quantityInput,
			'unavailable_message' => __('This product is currently out of stock and unavailable.', 'woocommerce'),
		]);

		echo $this->getView()->render('@woocommerce/single-product/add-to-cart/variable.twig', $templateVars);
	}

	/**
	 * Renderira djecu grupiranog prozvoda.
	 *
	 * @param WC_Product_Grouped $grouped
	 *
	 * @return string
	 */
	protected function renderGroupedChildren(WC_Product_Grouped $grouped)
	{
		$children = [];

		foreach ($grouped->get_children() as $childId) {
			$child = wc_get_product($childId);
			$children[] = $this->renderGroupedChild($child);
		}

		return implode("\n", $children);
	}

	/**
	 * Renderira jedno dijete grupiranog proizvoda.
	 *
	 * @param WC_Product $child
	 *
	 * @return string
	 */
	protected function renderGroupedChild(WC_Product $child)
	{
		ob_start();
		woocommerce_quantity_input([
			'input_name'  => 'quantity[' . $child->get_id() . ']',
			'input_value' => isset($_POST['quantity'][$child->get_id()]) ? wc_stock_amount($_POST['quantity'][$child->get_id()]) : 0,
			'min_value'   => apply_filters('woocommerce_quantity_input_min', 0, $child),
			'max_value'   => apply_filters('woocommerce_quantity_input_max', $child->get_max_purchase_quantity(), $child),
		]);
		$quantityInput = ob_get_clean();

		if ($child->is_visible()) {
			$productLabel = sprintf(
				'<a href="%s">%s</a>',
				esc_url(apply_filters('woocommerce_grouped_product_list_link', get_permalink($child->get_id()), $child->get_id())),
				$child->get_name()
			);
		} else {
			$productLabel = $child->get_name();
		}

		$templateVars = ArrayUtils::merge($this->getTemplateVars($child), [
			'product_label' => $productLabel,
			'quantity_input' => $quantityInput,
		]);

		return $this->getView()->render('@woocommerce/single-product/add-to-cart/grouped-child.twig', $templateVars);
	}

	/**
	 * Renderira sve atribute varijabilnog proizvoda.
	 *
	 * @param WC_Product_Variable $product
	 *
	 * @return string
	 */
	protected function renderVariableAttributes(WC_Product_Variable $product)
	{
		$attributes = $product->get_variation_attributes();
		$attributeKeys = array_keys($attributes);
		$last = end($attributeKeys);
		$rendered = [];

		foreach ($attributes as $name => $values) {
			$rendered[] = $this->renderVariableAttribute($product, $name, $values, $last === $name);
		}

		return implode("\n", $rendered);
	}

	/**
	 * Renderira jedan atribut varijabilnog proizvoda.
	 *
	 * @param WC_Product_Variable $product
	 * @param string $name
	 * @param array $values
	 *
	 * @param bool $isLast
	 *
	 * @return string
	 */
	protected function renderVariableAttribute(WC_Product_Variable $product, $name, array $values, $isLast = false)
	{
		$slug = sanitize_title($name);

		$selected = isset($_REQUEST['attribute_' . $slug])
			? wc_clean(stripslashes(urldecode($_REQUEST['attribute_' . $slug])))
			: $product->get_variation_default_attribute($name);

		ob_start();
		wc_dropdown_variation_attribute_options([
			'options' => $values,
			'attribute' => $name,
			'product' => $product,
			'selected' => $selected
		]);
		$dropdown = ob_get_clean();

		return $this->getView()->render('@woocommerce/single-product/add-to-cart/variable-attribute.twig', [
			'dropdown' => $dropdown,
			'is_last' => $isLast,
			'label' => wc_attribute_label($name),
			'reset_text' => __('Clear', 'woocommerce'),
			'slug' => $slug,
		]);
	}

	/**
	 * Priprema zajedničke varijable za templateove.
	 *
	 * @param WC_Product $product
	 *
	 * @return array
	 */
	protected function getTemplateVars(WC_Product $product)
	{
		ob_start();
		wc_print_notices();
		$notices = ob_get_clean();

		if ($product->is_visible()) {
			$productLabel = sprintf(
				'<a href="%s">%s</a>',
				esc_url(get_permalink($product->get_id())),
				$product->get_name()
			);
		} else {
			$productLabel = $product->get_name();
		}

		if ($product instanceof WC_Product_Grouped) {
			$purchasable = false;
			/** @var WC_Product[] $children */
			$children = array_map('wc_get_product', $product->get_children(''));

			foreach ($children as $child) {
				if ($child->is_purchasable()) {
					$purchasable = true;
					break;
				}
			}
		} else {
			$purchasable = $product->is_purchasable();
		}

		return [
			'button_text' => $product->single_add_to_cart_text(),
			'css_class' => implode(' ', get_post_class('', $product->get_id())),
			'in_stock' => $product->is_in_stock(),
			'is_purchasable' => $purchasable,
			'notices' => $notices,
			'product_id' => $product->get_id(),
			'product_label' => $productLabel,
			'product_price' => $product->get_price_html(),
			'product_sku' => $product->get_sku(),
			'stock_html' => wc_get_stock_html($product),
		];
	}
}